local L = AceLibrary("AceLocale-2.2"):new("EpicCountFu")
local Tablet = AceLibrary("Tablet-2.0")

local options = {
	handler = EpicCountFu,
	type = "group",
	args = {
		sep1 = {
			type = "header",
			order = 1.2,
		},
		emptyslots = {
			type = "toggle",
			order = 1.21,
			name = L["Empty Slots"],
			desc = L["Include Empty Slots"],
			get = "IsShowEmptySlots",
			set = "ToggleShowEmptySlots",
		},
  	inspectui = {
  		type = "toggle",
  		order = 1.22,
  		name = L["Show in Inspect"],
  		desc = L["Show the count in the inspect UI"],
  		get = "IsShowInspect",
  		set = "ToggleShowInspect",
  	},
		advanced = {
			type = "group",
			order = 99,
			name = L["Advanced Options"],
			desc = L["Advanced Options"],
			args = {
				smartexclude = {
					type = "toggle",
					order = 1.5,
					name = L["\"Smart\" Exclude"],
					desc = L["\"Smart\" Exclude"],
					get = "IsSmartExclude",
					set = "ToggleSmartExclude",
				}
			}
		}
	},
}

EpicCountFu = AceLibrary("AceAddon-2.0"):new(
  "AceEvent-2.0","AceConsole-2.0","AceDB-2.0",
  "FuBarPlugin-2.0","AceDebug-2.0","AceHook-2.1")

EpicCountFu.version = "$Revision: 58266 $"
EpicCountFu.hasIcon = "Interface\\Icons\\INV_Elemental_Primal_Mana"
EpicCountFu.clickableTooltip = true
EpicCountFu.independentProfile = true
EpicCountFu.tooltipHiddenWhenEmpty = true
EpicCountFu.debugFrame = ChatFrame5

EpicCountFu:RegisterChatCommand({"/epiccount", "/epicount"}, options)
EpicCountFu.OnMenuRequest = options

function EpicCountFu:OnClick()
	self:Update()
end

EpicCountFu:RegisterDB("EpicCountFuDB")
EpicCountFu:RegisterDefaults("profile", {
	emptyslots = false,
	optional_slots = false,
	smartexclude = true,
	inspect = true,
})

local slots = {
	"HeadSlot",
	"NeckSlot",
	"ShoulderSlot",
	"BackSlot",
	"ChestSlot",
	"WristSlot",
	"HandsSlot", 
	"WaistSlot",
	"LegsSlot",
	"FeetSlot",
	"Finger0Slot",
	"Finger1Slot",
	"Trinket0Slot",
	"Trinket1Slot",
	"MainHandSlot",
	"SecondaryHandSlot",
	"RangedSlot",
}

local optional_slots = {
	"ShirtSlot",
	"TabardSlot",
}


local InspectFrameSafeToOverride = false;

function EpicCountFu:OnInitialize()
end

function EpicCountFu:OnEnable()
	self:RegisterEvent("UNIT_STATS", function() self:Update() end)
	self:RegisterEvent("ADDON_LOADED");
	self:Update()
end

function EpicCountFu:ADDON_LOADED(arg1)
	if (arg1 == "FuBar_EpicCountFu") then
		if (InspectFrame_Show) then
			self:CheckInspect();
		end
	elseif (arg1 == "Blizzard_InspectUI") then
		self:CheckInspect();
	end
end

local origText;
function EpicCountFu:CheckInspect()
  if not InspectFrameSafeToOverride then
    origText = InspectTitleText:GetText();
    if (InspectTitleText:GetText() == "title of the player") then
      InspectFrameSafeToOverride = true;
    end
  end
  if InspectFrameSafeToOverride and self.db.profile.inspect then
    self:SecureHook("InspectPaperDollFrame_OnShow","CountInspectEpics")
  end
end

function EpicCountFu:OnProfileEnable()
	self:Update()
end

EpicCountFu._data = {
	[6] = 0,
	[5] = 0,
	[4] = 0,
	[3] = 0,
	[2] = 0,
	[1] = 0,
	[0] = 0,
	empty = 0,
	empties = {},
	slots = {
		[6] = {},
		[5] = {},
		[4] = {},
		[3] = {},
		[2] = {},
		[1] = {},
		[0] = {},
	}
}
EpicCountFu._slots = {}

EpicCountFu._idata = {
	[6] = 0,
	[5] = 0,
	[4] = 0,
	[3] = 0,
	[2] = 0,
	[1] = 0,
	[0] = 0,
	empty = 0,
	empties = {},
	slots = {
		[6] = {},
		[5] = {},
		[4] = {},
		[3] = {},
		[2] = {},
		[1] = {},
		[0] = {},
	}
}
EpicCountFu._islots = {}

local function wipeTables(_data,_slots)
  for k in pairs(_slots) do
    local kslots = _slots[k];
    for l in pairs(kslots) do
      kslots[l] = nil;
    end
  end
  for i=0,6 do
    _data[i] = 0;
    local dslots = _data.slots[i];
    for k in pairs(dslots) do
      dslots[k] = nil
    end
  end
  _data.empty = 0;
  for k in pairs(_data.empties) do
    _data.empties[k] = nil
  end
end

function EpicCountFu:OnDataUpdate()
  wipeTables(self._data,self._slots);
	if (self.db.profile.smartexclude) then
		self:SE_run(self._data,self._slots,"player");
	else
		self:runSlotFor(self._data,self._slots,"player",slots);
		if (self.db.profile.optional_slots) then
			self:runSlotFor(self._data,self._slots,"player",optional_slots);
		end
	end
end

function EpicCountFu:runSlotFor(_data,_slots,unit,slots)
	for _,slotname in pairs(slots) do
		self:doCount(_data,_slots,unit,self:process(slotname))
	end
end

function EpicCountFu:OnTextUpdate()
	local text = "";
	for ri = 0, 6 do
		local i = 6 - ri;
		if (self._data[i] ~= 0) then
			local r, g, b, hex = GetItemQualityColor(i);
			text = text .. " " .. hex .. self._data[i] .. "|r"
		end
	end
	if (self.db.profile.emptyslots and self._data.empty ~= 0) then
		local r, g, b, hex = GetItemQualityColor(0);
		text = text .. " E:" .. hex .. self._data.empty .. "|r"
	end
	self:SetText(text);
end

function EpicCountFu:OnTooltipUpdate()
	for ri = 0, 6 do
		local i = 6 - ri;
		if (self._data[i] ~= 0) then
			local r, g, b, hex = GetItemQualityColor(i);
			local category = Tablet:AddCategory(
				"id", "rarity" .. i
			)
			local items = {}
			for _,slotname in pairs(self._data.slots[i]) do
				local slotdata = self:process(slotname);
				tinsert(items, slotdata);
			end
			sort(items,function(a,b) return a.itemName<b.itemName end);
			
			for _,slotData in ipairs(items) do
				category:AddLine(
					'text', slotData.itemName,
					'textR', r,
					'textG', g,
					'textB', b
				)
			end
		end
	end
	
	if (self.db.profile.emptyslots and self._data.empty ~= 0) then
		local r, g, b, hex = GetItemQualityColor(0);
		local category = Tablet:AddCategory(
			"id", "empty"
		)
		
		category:AddLine(
			'text', self._data.empty .. " empty slot(s)",
			'textR', r,
			'textG', g,
			'textB', b
		)
		for _,slotname in pairs(self._data.empties) do
			category:AddLine(
				'text',slotname,
				'textR', r,
				'textG', g,
				'textB', b
			)
		end
	end
end

function EpicCountFu:process(_data,_slots,unit,slot)
  if (_slots == nil) then return; end
	if (_slots[slot] == nil) then
    _slots[slot] = {};
  end
  if (_slots[slot].empty == nil) then
		local itemLink = GetInventoryItemLink(unit, GetInventorySlotInfo(slot) )
		if (itemLink == nil) then
			local s = _slots[slot];
			s.empty = true
			s.slotName = slot
		else
			local itemName, _, itemRarity, _, _, itemType, itemSubType, _, _, _ = GetItemInfo(itemLink);
			local s = _slots[slot]
			s.empty = false
			s.slotName = slot
			s.itemLink = itemLink
			s.itemName = itemName
			s.itemRarity = itemRarity
			s.itemType = itemType
			s.itemSubType = itemSubType
		end
	end
	return _slots[slot];
end

function EpicCountFu:doCount(_data,_slots,unit,slotdata)
	if (slotdata.empty or slotdata.itemRarity == nil or slotdata.itemRarity < 0 or slotdata.itemRarity > 6) then
		_data.empty = _data.empty + 1;
		_data.empties[_data.empty] = slotdata.slotName;
	else
		local idx = _data[slotdata.itemRarity] + 1;
		_data[slotdata.itemRarity] = idx;
		_data.slots[slotdata.itemRarity][idx] = slotdata.slotName;
	end
end

function EpicCountFu:CountInspectEpics()
  wipeTables(self._idata,self._islots);
  if (self.db.profile.smartexclude) then
  	self:SE_run(self._idata,self._islots,InspectFrame.unit);
  else
  	self:runSlotFor(self._idata,self._islots,InspectFrame.unit,slots);
  	if (self.db.profile.optional_slots) then
  		self:runSlotFor(self._idata,self._islots,InspectFrame.unit,optional_slots);
  	end
  end
  
  local text = "";
	for ri = 0, 6 do
		local i = 6 - ri;
		if (self._idata[i] ~= 0) then
			local r, g, b, hex = GetItemQualityColor(i);
			text = text .. " " .. hex .. self._idata[i] .. "|r"
		end
	end
	if (self.db.profile.emptyslots and self._idata.empty ~= 0) then
		local r, g, b, hex = GetItemQualityColor(0);
		text = text .. " E:" .. hex .. self._idata.empty .. "|r"
	end
	
	InspectTitleText:SetText(text);
	InspectTitleText:Show();
end

function EpicCountFu:IsShowEmptySlots()
	return self.db.profile.emptyslots
end

function EpicCountFu:ToggleShowEmptySlots()
	self.db.profile.emptyslots = not self.db.profile.emptyslots
	self:Update()
	return self.db.profile.emptyslots
end

function EpicCountFu:IsShowInspect()
	return self.db.profile.inspect
end

function EpicCountFu:ToggleShowInspect()
	self.db.profile.inspect = not self.db.profile.inspect
	if (self.db.profile.inspect) then
	  self:SecureHook("InspectPaperDollFrame_OnShow","CountInspectEpics");
	  if (InspectTitleText) then
	    InspectTitleText:Show();
	  end
	  if (InspectFrame:IsShown()) then
	    self:CountInspectEpics()
	  end
	else
	  self:Unhook("InspectPaperDollFrame_OnShow");
	  if (InspectTitleText) then
	    InspectTitleText:Hide();
	  end
	end
	return self.db.profile.inspect
end

function EpicCountFu:IsSmartExclude()
	return self.db.profile.smartexclude
end

function EpicCountFu:ToggleSmartExclude()
	self.db.profile.smartexclude = not self.db.profile.smartexclude
	self:Update()
	return self.db.profile.smartexclude
end