local B = AceLibrary("LibBabble-Inventory-3.0");
local BI = B:GetLookupTable();

local _data;
local _slots;
local _unit;

local function se_function_level(_level)
	return function (self,slot,unit)
		local level = UnitLevel(_unit);
		return not slot.empty or level >= _level;
	end
end

local function se_function_levelother(_level,_other)
	return function (self,slot,unit)
		local other = self:process(_data,_slots,_unit,_other);
		local level = UnitLevel(_unit);
		return not slot.empty or (level < _level and not other.empty) or level >= _level;
	end
end

-- For Smart Exclude
local se_slot_data = {
	["HeadSlot"] = se_function_level(30),
	["NeckSlot"] = se_function_level(15),
	["ShoulderSlot"] = se_function_level(18),
	["BackSlot"] = se_function_level(10),
	["ChestSlot"] = true,
	["WristSlot"] = true,
	["HandsSlot"] = true, 
	["WaistSlot"] = true,
	["LegsSlot"] = true,
	["FeetSlot"] = true,
	
	["Finger0Slot"] = se_function_levelother(15,"Finger1Slot"),
	["Finger1Slot"] = se_function_levelother(15,"Finger0Slot"),
	
	["Trinket0Slot"] = se_function_levelother(30,"Trinket1Slot"),
	["Trinket1Slot"] = se_function_levelother(30,"Trinket0Slot"),
	["MainHandSlot"] = true,
	["SecondaryHandSlot"] = function (self,slot,unit)
		local _, englishClass = UnitClass(unit);
		local mh = self:process(_data,_slots,_unit,"MainHandSlot");
		local st = mh.itemSubType;
		if (not slot.empty) then return true; end
		if (st == BI["Staves"] or
			st == BI["Two-Handed Axes"] or
			st == BI["Two-Handed Maces"] or
			st == BI["Two-Handed Swords"]) then return false; end
		return true;
	end,
	["RangedSlot"] = function (self,slot,unit)
		local _, englishClass = UnitClass(unit);
		local level = UnitLevel(unit);
		if (not slot.empty) then return true; end
		if (englishClass == "DRUID") then return level >= 50; end
		if (englishClass == "PALADIN") then return level >= 50; end
		if (englishClass == "SHAMAN") then return level >= 60; end
		if (englishClass == "DEATHKNIGHT") then return level >= 55; end
		return true;
	end,
	["ShirtSlot"] = function (self,slot,unit)
		return (not slot.empty) and slot.itemRarity > 1
	end,
	["TabardSlot"] = function (self,slot,unit)
		return (not slot.empty) and slot.itemRarity > 1
	end,
	["AmmoSlot"] = function (self,slot,unit)
		return (not slot.empty) and slot.itemRarity > 2
	end,
}

function EpicCountFu:SE_run(data,slots,unit)
  if (data == nil) then return; end
  _data = data;
  _slots = slots;
  _unit = unit;
	for slotname,slotfunc in pairs(se_slot_data) do
		local slotdata = self:process(_data,_slots,_unit,slotname);
		if ((type(slotfunc) == "function" and slotfunc(self,slotdata,unit))
		 		or (type(slotfunc) ~= "function" and slotfunc)) then
			self:doCount(_data,_slots,_unit,slotdata);
		end
	end
end